/*
 * gc global variable provides access to GUI Composer infrastructure components and project information.
 * For more information, please see the Working with Javascript guide in the online help.
 */
var gc = gc || {};
gc.services = gc.services || {};

/*
 *  Boilerplate code for creating computed data bindings
 */
document.addEventListener('gc-databind-ready', function() {
    /* 
     *   Add custom computed value databindings here, using the following method:
     *
     *   function gc.databind.registry.bind(targetBinding, modelBinding, [getter], [setter]);
     *
     *   param targetBinding - single binding string or expression, or array of binding strings for multi-way binding.
     *   param modelBinding - single binding string or expression, or array of binding strings for multi-way binding.
     *   param getter - (optional) - custom getter function for computing the targetBinding value(s) based on modelBinding value(s).
     *   param setter - (optional) - custom setter function for computing the modelBinding value(s) based on targetBinding value(s).
     */

    // For example, a simple computed values based on simple expression
    // gc.databind.registry.bind('widget.id.propertyName', "targetVariable == 1 ? 'binding is one' : 'binding is not one'");

    // Or a custom two-way binding with custome getter and setter functions.  (setter is optional)  (getter only indicates one-way binding)
    // gc.databind.registry.bind('widget.id.propertyName', "targetVariable", function(value) { return value*5/9 + 32; }, function(value) { (return value-32)*9/5; });

    // Event 1 to n bindings
    /* 
    gc.databind.registry.bind('widget.date.value', 
        // dependant bindings needed in order to compute the date, in name/value pairs.
        {
            weekday: 'widget.dayOfWeek.selectedText',
            day: 'widget.dayOfMonth.value',
            month: 'widget.month.selectedText',
            year: 'widget.year.value'
        }, 
        // getter for date computation
        function(values) 
        {
            // compute and return the string value to bind to the widget with id 'date'
            return values.weekday + ', ' + values.month + ' ' + values.day + ', ' + values.year;
        }
    ); 
	*/
});

/*
 *  Boilerplate code for creating custom actions
 */
document.addEventListener('gc-nav-ready', function() {
    /* 
     *   Add custom actions for menu items using the following api:
     *
     *   function gc.nav.registryAction(id, runable, [isAvailable], [isVisible]);
     *
     *   param id - uniquely identifies the action, and should correspond to the action property of the menuaction widget.
     *   param runable - function that performs the custom action.
     *   param isAvailable - (optional) - function called when the menu action is about to appear.  Return false to disable the action, or true to enable it.
     *   param isVisible - (optional) - function called when the menu action is about to appear.  Return false to hide the action, or true to make it visible.
     */

    // For example,
    // gc.nav.registerAction('myCustomCloseAction', function() { window.close(); }, function() { return true; }, function() { return true; });

    // Alternatively, to programmatically disable a menu action at any time use:
    // gc.nav.disableAction('myCustomCloseAction);    then enable it again using:  gc.nav.enableAction('myCustomCloseAction'); 
});

/*
 *  Boilerplate code for working with components in the application gist
 */


var initComplete = false;
var manifestUpdated = false;
var templateObj;
var backplaneEventHdlr = function(event) {
    var backplane = gc.services['ti-core-backplane'];
    if (backplane.isDisconnectBtnVisible) {
        backplane.removeEventListener('btnVisibilityChanged', backplaneEventHdlr)
        backplane.updateStateMachine('onFailedToConnect');
        window.setTimeout(function() {
            if (gc.connectionManager) {
                gc.connectionManager.stopBackplane();
            }
        }, 200); // wait for the backplaneConnectionCount to be set to 1, and then clear it
    }
};

// Wait for DOMContentLoaded event before trying to access the application template
var init = function() {
    templateObj = document.querySelector('#template_obj');
    // Wait for the template to fire a dom-change event to indicate that it has been 'stamped'
    // before trying to access components in the application.
    templateObj.addEventListener('dom-change', function() {
        if (initComplete) return;
        initComplete = true;
        
        // Code to configure the icon in the browser tab: [...
        var link = document.createElement('link');
        link.type = 'image/png';
        link.rel = 'shortcut icon';
        link.href = 'images/ti_gui_icon.png';
        document.getElementsByTagName('head')[0].appendChild(link);
        // ..]

        gc.fileCache.readJsonFile('project.json').then(function(manifest) {
            manifest.disableAutoConnect = true;
            return gc.fileCache.writeJsonFile('project.json', manifest);
        }).then(function() {
            manifestUpdated = true;

            var backplane = gc.services['ti-core-backplane'];
            backplane.addEventListener('btnVisibilityChanged', backplaneEventHdlr);
        });

        this.async(function() {
            console.log("Application template has been stamped.");
            // Now that the template has been stamped, you can use 'automatic node finding' $ syntax to access widgets.
            // e.g. to access a widget with an id of 'widget_id' you can use templateObj.$.widgetId
            templateObj.$.ti_widget_filtered_droplist.addEventListener('selected', function(event) {
                var deviceName = templateObj.$.ti_widget_filtered_droplist.selectedText;
                var outFileName = templateObj.$.ti_widget_filtered_droplist.selectedValue;
                var model = document.querySelector('ti-program-model');
                model.deviceName = deviceName;
                model.targetProgram = outFileName;
            });
            templateObj.$.ti_widget_button.addEventListener('click', function(event) {
                var dlg = document.querySelector('ti-service-programloader-dialog');
                dlg.open();
            });
            templateObj.$.ti_widget_checkbox.addEventListener('changed', function(event) {
                var model = document.querySelector('ti-program-model');
                model.autoLoadProgram = templateObj.$.ti_widget_checkbox.checked;
                templateObj.$.ti_widget_button.disabled = templateObj.$.ti_widget_checkbox.checked;
            })

        }, 1);

    });
};

templateObj = document.querySelector('#template_obj');
if (templateObj) {
    init();
} else {
    document.addEventListener('DOMContentLoaded', init.bind(this))
}